@@ -74,9 +74,6 @@ import javax.mail.internet.idn.IDNA;
 import javax.security.auth.Subject;
 import org.cliffc.high_scale_lib.NonBlockingHashMap;
 import com.openexchange.config.ConfigurationService;
-import com.openexchange.config.Interests;
-import com.openexchange.config.Reloadable;
-import com.openexchange.config.Reloadables;
 import com.openexchange.exception.OXException;
 import com.openexchange.imap.acl.ACLExtension;
 import com.openexchange.imap.acl.ACLExtensionInit;
@@ -87,7 +84,6 @@ import com.openexchange.imap.cache.RootSubfoldersEnabledCache;
 import com.openexchange.imap.config.IIMAPProperties;
 import com.openexchange.imap.config.IMAPConfig;
 import com.openexchange.imap.config.IMAPProperties;
-import com.openexchange.imap.config.IMAPReloadable;
 import com.openexchange.imap.config.IMAPSessionProperties;
 import com.openexchange.imap.config.MailAccountIMAPProperties;
 import com.openexchange.imap.converters.IMAPFolderConverter;
@@ -95,26 +91,22 @@ import com.openexchange.imap.entity2acl.Entity2ACLInit;
 import com.openexchange.imap.ping.IMAPCapabilityAndGreetingCache;
 import com.openexchange.imap.services.Services;
 import com.openexchange.imap.storecache.IMAPStoreCache;
-import com.openexchange.imap.storecache.IMAPStoreContainer;
 import com.openexchange.imap.util.HostAndPort;
 import com.openexchange.imap.util.HostAndPortAndCredentials;
 import com.openexchange.imap.util.StampAndOXException;
+import com.openexchange.java.Streams;
 import com.openexchange.java.Strings;
 import com.openexchange.log.LogProperties;
 import com.openexchange.log.audit.AuditLogService;
 import com.openexchange.log.audit.DefaultAttribute;
 import com.openexchange.log.audit.DefaultAttribute.Name;
 import com.openexchange.mail.MailExceptionCode;
-import com.openexchange.mail.Protocol;
 import com.openexchange.mail.api.AuthType;
-import com.openexchange.mail.api.IMailFolderStorage;
-import com.openexchange.mail.api.IMailMessageStorage;
 import com.openexchange.mail.api.IMailProperties;
 import com.openexchange.mail.api.IMailStoreAware;
 import com.openexchange.mail.api.MailAccess;
 import com.openexchange.mail.api.MailConfig;
 import com.openexchange.mail.api.MailLogicTools;
-import com.openexchange.mail.cache.IMailAccessCache;
 import com.openexchange.mail.config.MailProperties;
 import com.openexchange.mail.config.MailProxyConfig;
 import com.openexchange.mail.dataobjects.MailFolder;
@@ -257,16 +249,6 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
      */
     private transient Subject kerberosSubject;
 
-    /**
-     * The IMAP protocol.
-     */
-    private final Protocol protocol;
-
-    /**
-     * The max. connection count.
-     */
-    private int maxCount;
-
     /**
      * The connected flag.
      */
@@ -300,30 +282,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
     /**
      * The IMAP configuration.
      */
-    private volatile IMAPConfig imapConfig;
-
-    /**
-     * A simple cache for max. count values per server.
-     */
-    private static volatile ConcurrentMap<String, Integer> maxCountCache;
-
-    static {
-        IMAPReloadable.getInstance().addReloadable(new Reloadable() {
-
-            @Override
-            public void reloadConfiguration(final ConfigurationService configService) {
-                final ConcurrentMap<String, Integer> m = maxCountCache;
-                if (null != m) {
-                    m.clear();
-                }
-            }
-
-            @Override
-            public Interests getInterests() {
-                return Reloadables.interestsForProperties("com.openexchange.imap.maxNumConnections");
-            }
-        });
-    }
+    private transient volatile IMAPConfig imapConfig;
 
     /**
      * Initializes a new {@link IMAPAccess IMAP access} for default IMAP account.
@@ -333,8 +292,6 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
     protected IMAPAccess(final Session session) {
         super(session);
         setMailProperties((Properties) System.getProperties().clone());
-        maxCount = -1;
-        protocol = IMAPProvider.PROTOCOL_IMAP;
     }
 
     /**
@@ -346,30 +303,6 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
     protected IMAPAccess(final Session session, final int accountId) {
         super(session, accountId);
         setMailProperties((Properties) System.getProperties().clone());
-        maxCount = -1;
-        protocol = IMAPProvider.PROTOCOL_IMAP;
-    }
-
-    /**
-     * Gets the max. count for this IMAP access.
-     *
-     * @return The max. count
-     * @throws OXException If an exception occurs
-     */
-    protected int getMaxCount() throws OXException {
-        final ConcurrentMap<String, Integer> cache = maxCountCache;
-        if (null == cache) {
-            return protocol.getMaxCount(server, MailAccount.DEFAULT_ID == accountId);
-        }
-        Integer maxCount = cache.get(server);
-        if (null == maxCount) {
-            final Integer i = Integer.valueOf(protocol.getMaxCount(server, MailAccount.DEFAULT_ID == accountId));
-            maxCount = cache.putIfAbsent(server, i);
-            if (null == maxCount) {
-                maxCount = i;
-            }
-        }
-        return maxCount.intValue();
     }
 
     @Override
@@ -524,7 +457,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         if (null == tmp) {
             try {
                 return (IMAPConfig) getMailConfig();
-            } catch (final OXException e) {
+            } catch (@SuppressWarnings("unused") OXException e) {
                 // Cannot occur
                 return null;
             }
@@ -634,7 +567,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
              */
             javax.mail.Session imapSession;
             {
-                boolean forceSecure = imapConfig.isRequireTls();
+                boolean forceSecure = getIMAPConfig().isRequireTls();
                 imapSession = setConnectProperties(config, imapConfProps.getImapTimeout(), imapConfProps.getImapConnectionTimeout(), imapProps, JavaIMAPStore.class, forceSecure, session.getUserId(), session.getContextId());
             }
             /*
@@ -663,7 +596,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                             warnings.add(MailExceptionCode.NON_SECURE_WARNING.create());
                         }
                     }
-                } catch (final MessagingException e) {
+                } catch (@SuppressWarnings("unused") MessagingException e) {
                     // Ignore
                 }
             } catch (final AuthenticationFailedException e) {
@@ -773,7 +706,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
              * Remove proxy settings for whitelisted hosts
              */
             HostList nonProxyHosts = MailProxyConfig.getInstance().getImapNonProxyHostList();
-            if (nonProxyHosts.contains(imapConfig.getImapServerAddress())) {
+            if (nonProxyHosts.contains(getIMAPConfig().getImapServerAddress())) {
                 imapProps.remove("mail.imap.proxy.host");
                 imapProps.remove("mail.imap.proxy.port");
                 imapProps.remove("mail.imaps.proxy.host");
@@ -785,7 +718,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
              */
             {
                 final Class<? extends IMAPStore> clazz = useIMAPStoreCache() ? IMAPStoreCache.getInstance().getStoreClass() : JavaIMAPStore.class;
-                boolean forceSecure = accountId > 0 && imapConfig.isRequireTls();
+                boolean forceSecure = accountId > 0 && getIMAPConfig().isRequireTls();
                 imapSession = setConnectProperties(config, imapConfProps.getImapTimeout(), imapConfProps.getImapConnectionTimeout(), imapProps, clazz, forceSecure, session.getUserId(), session.getContextId());
             }
             /*
@@ -818,11 +751,13 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
              */
             checkAuthFailed(this.login, this.password, imapConfProps);
             this.clientIp = clientIp;
-            maxCount = getMaxCount();
+            int maxCount = getIMAPConfig().getIMAPProperties().getMaxNumConnection();
             try {
                 imapStore = connectIMAPStore(maxCount);
             } catch (final AuthenticationFailedException e) {
                 if (accountId != MailAccount.DEFAULT_ID) {
+                    int accountId = this.accountId;
+                    Session session = this.session;
                     AbstractTask<Void> task = new AbstractTask<Void>() {
 
                         @Override
@@ -898,7 +833,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                             try {
                                 dummy.myRights();
                                 nb = Boolean.TRUE;
-                            } catch (MessagingException e) {
+                            } catch (@SuppressWarnings("unused") MessagingException e) {
                                 // MessagingException - If the server doesn't support the ACL extension
                                 nb = Boolean.FALSE;
                             }
@@ -946,25 +881,6 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         return connectIMAPStore(maxCount, imapSession, server, port, login, password, clientIp);
     }
 
-    /**
-     * Clears cached IMAP connections.
-     */
-    protected void clearCachedConnections() {
-        final IMAPStoreContainer container = IMAPStoreCache.getInstance().optContainer(accountId, server, port, login, session);
-        if (null != container) {
-            container.clear();
-        }
-        try {
-            final IMailAccessCache mailAccessCache = MailAccess.getMailAccessCache();
-            MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> tmp;
-            while ((tmp = mailAccessCache.removeMailAccess(session, accountId)) != null) {
-                tmp.close(false);
-            }
-        } catch (final Exception e) {
-            // Ignore
-        }
-    }
-
     private static final String PROTOCOL = IMAPProvider.PROTOCOL_IMAP.getName();
 
     private IMAPStore connectIMAPStore(final int maxCount, final javax.mail.Session imapSession, final String server, final int port, final String login, final String pw, final String clientIp) throws MessagingException, OXException {
@@ -979,9 +895,10 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         /*
          * Set log properties
          */
+        IMAPConfig config = getIMAPConfig();
         LogProperties.put(LogProperties.Name.MAIL_ACCOUNT_ID, Integer.valueOf(accountId));
         LogProperties.put(LogProperties.Name.MAIL_HOST, server + ":" + port);
-        LogProperties.put(LogProperties.Name.MAIL_LOGIN, imapConfig.getLogin());
+        LogProperties.put(LogProperties.Name.MAIL_LOGIN, config.getLogin());
         /*-
          * Get connected IMAP store
          *
@@ -1000,7 +917,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
             boolean checkConnectivityIfPolled = checkConnectivityIfPolled();
             final IMAPStore borrowedIMAPStore = borrowIMAPStore(imapSession, server, port, login, pw, (clientIp != null), checkConnectivityIfPolled);
             if (null == borrowedIMAPStore) {
-                throw IMAPException.create(IMAPException.Code.CONNECTION_UNAVAILABLE, imapConfig, session, imapConfig.getServer(), imapConfig.getLogin());
+                throw IMAPException.create(IMAPException.Code.CONNECTION_UNAVAILABLE, config, session, config.getServer(), config.getLogin());
             }
             return borrowedIMAPStore;
         }
@@ -1029,7 +946,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                 }
             }
         }
-        throw new MessagingException("Unable to connect to IMAP store: " + new URLName("imap", server, port, null, imapConfig.getLogin(), "xxxx"));
+        throw new MessagingException("Unable to connect to IMAP store: " + new URLName("imap", server, port, null, config.getLogin(), "xxxx"));
     }
 
     private IMAPStore newConnectedImapStore(javax.mail.Session imapSession, String server, int port, String login, String pw, int accountId) throws MessagingException {
@@ -1042,7 +959,17 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
          */
         IMAPStore imapStore = (IMAPStore) imapSession.getStore(PROTOCOL);
         if (MailAccount.DEFAULT_ID == accountId) {
-            IMAPClientParameters.setDefaultClientParameters(imapStore, session);
+            boolean failedToSetClientParameters = true; // Pessimistic
+            try {
+                IMAPClientParameters.setDefaultClientParameters(imapStore, session);
+                failedToSetClientParameters = false;
+            } catch (OXException e) {
+                throw new MessagingException(e.getMessage(), e);
+            } finally {
+                if (failedToSetClientParameters) {
+                    Streams.close(imapStore);
+                }
+            }
         }
         /*
          * ... and connect it
@@ -1080,28 +1007,34 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
      * @throws MessagingException If operation fails
      */
     public static void doIMAPConnect(javax.mail.Session imapSession, IMAPStore imapStore, String server, int port, String login, String pw, int accountId, Session session, boolean knownExternal) throws MessagingException {
-        Object kerberosSubject = imapSession.getProperties().get("mail.imap.sasl.kerberosSubject");
-        if (null == kerberosSubject) {
-            imapStore.connect(server, port, login, pw);
-        } else {
-            synchronized (kerberosSubject) {
+        boolean error = true;
+        try {
+            Object kerberosSubject = imapSession.getProperties().get("mail.imap.sasl.kerberosSubject");
+            if (null == kerberosSubject) {
                 imapStore.connect(server, port, login, pw);
+            } else {
+                synchronized (kerberosSubject) {
+                    imapStore.connect(server, port, login, pw);
+                }
+            }
+            AuditLogService auditLogService = Services.optService(AuditLogService.class);
+            if (null != auditLogService) {
+                String eventId = knownExternal ? "imap.external.login" : (MailAccount.DEFAULT_ID == accountId ? "imap.primary.login" : "imap.external.login");
+                auditLogService.log(eventId, DefaultAttribute.valueFor(Name.LOGIN, session.getLoginName()), DefaultAttribute.valueFor(Name.IP_ADDRESS, session.getLocalIp()), DefaultAttribute.timestampFor(new Date()), DefaultAttribute.arbitraryFor("imap.login", login), DefaultAttribute.arbitraryFor("imap.server", server), DefaultAttribute.arbitraryFor("imap.port", Integer.toString(port)));
+            }
+            String sessionInformation = imapStore.getClientParameter(IMAPClientParameters.SESSION_ID.getParamName());
+            if (null != sessionInformation) {
+                LogProperties.put(LogProperties.Name.MAIL_SESSION, sessionInformation);
+            }
+            java.net.InetAddress remoteAddress = imapStore.getRemoteAddress();
+            if (null != remoteAddress) {
+                LogProperties.put(LogProperties.Name.MAIL_HOST_REMOTE_ADDRESS, remoteAddress.getHostAddress());
+            }
+            error = false;
+        } finally {
+            if (error) {
+                Streams.close(imapStore);
             }
-        }
-
-        AuditLogService auditLogService = Services.optService(AuditLogService.class);
-        if (null != auditLogService) {
-            String eventId = knownExternal ? "imap.external.login" : (MailAccount.DEFAULT_ID == accountId ? "imap.primary.login" : "imap.external.login");
-            auditLogService.log(eventId, DefaultAttribute.valueFor(Name.LOGIN, session.getLoginName()), DefaultAttribute.valueFor(Name.IP_ADDRESS, session.getLocalIp()), DefaultAttribute.timestampFor(new Date()), DefaultAttribute.arbitraryFor("imap.login", login), DefaultAttribute.arbitraryFor("imap.server", server), DefaultAttribute.arbitraryFor("imap.port", Integer.toString(port)));
-        }
-
-        String sessionInformation = imapStore.getClientParameter(IMAPClientParameters.SESSION_ID.getParamName());
-        if (null != sessionInformation) {
-            LogProperties.put(LogProperties.Name.MAIL_SESSION, sessionInformation);
-        }
-        java.net.InetAddress remoteAddress = imapStore.getRemoteAddress();
-        if (null != remoteAddress) {
-            LogProperties.put(LogProperties.Name.MAIL_HOST_REMOTE_ADDRESS, remoteAddress.getHostAddress());
         }
     }
 
@@ -1231,7 +1164,6 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         RootSubfoldersEnabledCache.init();
         ACLExtensionInit.getInstance().start();
         Entity2ACLInit.getInstance().start();
-        maxCountCache = new NonBlockingHashMap<>(16);
 
         final ConfigurationService confService = Services.getService(ConfigurationService.class);
         final boolean useIMAPStoreCache = null == confService ? true : confService.getBoolProperty("com.openexchange.imap.useIMAPStoreCache", true);
@@ -1285,7 +1217,6 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
     @Override
     protected void shutdown() throws OXException {
         USE_IMAP_STORE_CACHE.set(true);
-        maxCountCache = null;
         Entity2ACLInit.getInstance().stop();
         ACLExtensionInit.getInstance().stop();
         IMAPCapabilityAndGreetingCache.tearDown();
@@ -1532,7 +1463,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                 i += b;
             }
             return (i < 0) ? -i : i;
-        } catch (NoSuchAlgorithmException e) {
+        } catch (@SuppressWarnings("unused") NoSuchAlgorithmException e) {
             // Ignore
         }
         return 0;
@@ -1547,45 +1478,16 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         return "[not connected]";
     }
 
-    private static final int MAX_STACK_TRACE_ELEMENTS = 1000;
-
-    private static void appendStackTrace(final StackTraceElement[] trace, final String lineSeparator, final StringBuilder sb) {
-        if (null == trace) {
-            return;
-        }
-        final int length = (MAX_STACK_TRACE_ELEMENTS <= trace.length) ? MAX_STACK_TRACE_ELEMENTS : trace.length;
-        for (int i = 0; i < length; i++) {
-            final StackTraceElement ste = trace[i];
-            final String className = ste.getClassName();
-            if (null != className) {
-                sb.append("    at ").append(className).append('.').append(ste.getMethodName());
-                if (ste.isNativeMethod()) {
-                    sb.append("(Native Method)");
-                } else {
-                    final String fileName = ste.getFileName();
-                    if (null == fileName) {
-                        sb.append("(Unknown Source)");
-                    } else {
-                        final int lineNumber = ste.getLineNumber();
-                        sb.append('(').append(fileName);
-                        if (lineNumber >= 0) {
-                            sb.append(':').append(lineNumber);
-                        }
-                        sb.append(')');
-                    }
-                }
-                sb.append(lineSeparator);
-            }
-        }
-    }
-
-    private static void closeSafely(final IMAPStore imapStore) {
+    /**
+     * Closes given IMAP store safely.
+     *
+     * @param imapStore The IMAP store to close
+     */
+    public static void closeSafely(final IMAPStore imapStore) {
         if (null != imapStore) {
             try {
                 imapStore.close();
-            } catch (final MessagingException e) {
-                LOG.error("Error while closing IMAP store.", e);
-            } catch (final RuntimeException e) {
+            } catch (Exception e) {
                 LOG.error("Error while closing IMAP store.", e);
             }
         }
