@@ -50,10 +50,10 @@
 package com.openexchange.imap.cache;
 
 import static com.openexchange.imap.IMAPCommandsCollection.performCommand;
-import gnu.trove.map.hash.TObjectIntHashMap;
+import static com.openexchange.java.Autoboxing.L;
+import java.io.IOException;
 import java.io.Serializable;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
@@ -72,22 +72,27 @@ import java.util.concurrent.atomic.AtomicReference;
 import javax.mail.Folder;
 import javax.mail.MessagingException;
 import org.cliffc.high_scale_lib.NonBlockingHashMap;
+import com.google.common.collect.ImmutableSet;
 import com.openexchange.exception.OXException;
 import com.openexchange.imap.IMAPCommandsCollection;
+import com.openexchange.java.ConcurrentHashSet;
 import com.openexchange.java.Strings;
 import com.openexchange.log.LogProperties;
 import com.openexchange.mail.mime.MimeMailException;
+import com.openexchange.mail.utils.MailFolderUtility;
 import com.sun.mail.iap.Argument;
 import com.sun.mail.iap.ProtocolException;
 import com.sun.mail.iap.Response;
 import com.sun.mail.imap.ACL;
 import com.sun.mail.imap.IMAPFolder;
 import com.sun.mail.imap.IMAPStore;
-import com.sun.mail.imap.Rights;
 import com.sun.mail.imap.protocol.BASE64MailboxDecoder;
 import com.sun.mail.imap.protocol.BASE64MailboxEncoder;
 import com.sun.mail.imap.protocol.IMAPProtocol;
 import com.sun.mail.imap.protocol.IMAPResponse;
+import com.sun.mail.imap.protocol.Namespaces;
+import com.sun.mail.imap.protocol.Namespaces.Namespace;
+import gnu.trove.map.hash.TObjectIntHashMap;
 
 /**
  * {@link ListLsubCollection}
@@ -108,6 +113,49 @@ final class ListLsubCollection implements Serializable {
         DEPRECATED, DEPRECATED_FORCE_NEW, INITIALIZED;
     }
 
+    /**
+     * For testing.
+     */
+    public static ListLsubCollection craftListLsubCollectionFrom(String rootList, String allList, String allLsub, String namespace) throws IOException, ProtocolException {
+        Response[] rootResponse = responsesFor(rootList);
+        Response[] allListResponse = responsesFor(allList);
+        Response[] allLsubResponse = responsesFor(allLsub);
+
+        String[] shared = null;
+        String[] user = null;
+        if (null != namespace) {
+            Response[] namespaceResponse = responsesFor(namespace);
+            Namespaces namespaces = new Namespaces(namespaceResponse[0]);
+
+            Namespace[] sharedNs = namespaces.shared;
+            if (null != sharedNs) {
+                shared = new String[sharedNs.length];
+                for (int i = 0; i < sharedNs.length; i++) {
+                    shared[i] = sharedNs[i].prefix;
+                }
+            }
+
+            Namespace[] userNs = namespaces.otherUsers;
+            if (null != userNs) {
+                user = new String[userNs.length];
+                for (int i = 0; i < userNs.length; i++) {
+                    user[i] = userNs[i].prefix;
+                }
+            }
+        }
+
+        return new ListLsubCollection(rootResponse, allListResponse, allLsubResponse, shared, user, false);
+    }
+
+    private static Response[] responsesFor(String rootList) throws IOException, ProtocolException {
+        String[] lines = rootList.split("\r?\n");
+        List<Response> responses = new ArrayList<>(lines.length);
+        for (String line : lines) {
+            responses.add(new IMAPResponse(line));
+        }
+        return responses.toArray(new Response[responses.size()]);
+    }
+
     // -----------------------------------------------------------------------------------------------------------
 
     final ConcurrentMap<String, ListLsubEntryImpl> listMap;
@@ -171,6 +219,44 @@ final class ListLsubCollection implements Serializable {
         init(false, imapStore, ignoreSubscriptions);
     }
 
+    /**
+     * Initializes a new {@link ListLsubCollection}.
+     */
+    private ListLsubCollection(Response[] rootResponse, Response[] allListResponses, Response[] allLsubResponses, String[] shared, String[] user, boolean ignoreSubscriptions) throws ProtocolException {
+        super();
+        listMap = new NonBlockingHashMap<String, ListLsubEntryImpl>();
+        lsubMap = new NonBlockingHashMap<String, ListLsubEntryImpl>();
+        draftsEntries = new ConcurrentHashMap<String, ListLsubEntry>();
+        junkEntries = new ConcurrentHashMap<String, ListLsubEntry>();
+        sentEntries = new ConcurrentHashMap<String, ListLsubEntry>();
+        trashEntries = new ConcurrentHashMap<String, ListLsubEntry>();
+        archiveEntries = new ConcurrentHashMap<String, ListLsubEntry>();
+        deprecated = new AtomicReference<State>();
+        this.shared = shared == null ? new String[0] : shared;
+        this.user = user == null ? new String[0] : user;
+
+        // Perform LIST "" ""
+        doRootListCommand(null, rootResponse);
+
+        // Perform LSUB "" "*"
+        if (!ignoreSubscriptions) {
+            doListLsubCommand(null, true, allLsubResponses);
+        }
+
+        // Perform LIST "" "*"
+        doListLsubCommand(null, false, allListResponses);
+
+        if (!ignoreSubscriptions) {
+            // Consistency check
+            checkConsistency(null);
+        }
+        /*
+         * Set time stamp
+         */
+        stamp = System.currentTimeMillis();
+        deprecated.set(State.INITIALIZED);
+    }
+
     @Override
     public String toString() {
         return toString(false);
@@ -463,15 +549,15 @@ final class ListLsubCollection implements Serializable {
             @Override
             public Object doCommand(final IMAPProtocol protocol) throws ProtocolException {
                 // Perform LIST "" ""
-                doRootListCommand(protocol);
+                doRootListCommand(protocol, null);
 
                 // Perform LSUB "" "*"
                 if (!ignoreSubscriptions) {
-                    doListLsubCommand(protocol, true);
+                    doListLsubCommand(protocol, true, null);
                 }
 
                 // Perform LIST "" "*"
-                doListLsubCommand(protocol, false);
+                doListLsubCommand(protocol, false, null);
 
                 if (ignoreSubscriptions) {
                     lsubMap.putAll(listMap);
@@ -518,7 +604,7 @@ final class ListLsubCollection implements Serializable {
 
         if (debug) {
             long dur = System.currentTimeMillis() - st;
-            LOG.debug("LIST/LSUB cache built in {}msec", dur);
+            LOG.debug("LIST/LSUB cache built in {}msec", L(dur));
         }
         /*
          * Set time stamp
@@ -557,7 +643,8 @@ final class ListLsubCollection implements Serializable {
          */
         for (Map.Entry<String, ListLsubEntryImpl> entry : new TreeMap<String, ListLsubEntryImpl>(lsubMap).entrySet()) {
             String fullName = entry.getKey();
-            if (!listMap.containsKey(fullName)) {
+            ListLsubEntryImpl listEntry = listMap.get(fullName);
+            if (null == listEntry) {
                 /*-
                  * An LSUB entry is not contained in LISTed ones...
                  *
@@ -581,6 +668,19 @@ final class ListLsubCollection implements Serializable {
                     final ListLsubEntryImpl lle = entry.getValue();
                     dropEntryFrom(lle, lsubMap);
                 }
+            } else if (listEntry.isDummy()) {
+                /*-
+                 * Corresponding LIST entry has been artificially added
+                 *
+                 * Check if it is a namespace folder w/o any subscribed child folders, if so remove it
+                 */
+                if (isNamespace(fullName)) {
+                    final ListLsubEntryImpl lle = entry.getValue();
+                    if (lle.emptyChildren()) {
+                        IMAPCommandsCollection.forceSetSubscribed(imapStore, fullName, false);
+                        dropEntryFrom(lle, lsubMap);
+                    }
+                }
             }
         }
     }
@@ -641,6 +741,9 @@ final class ListLsubCollection implements Serializable {
     }
 
     private static boolean existsSafe(final String fullName, final IMAPStore imapStore) {
+        if (null == imapStore) {
+            return false;
+        }
         try {
             return imapStore.getFolder(fullName).exists();
         } catch (final MessagingException e) {
@@ -740,21 +843,21 @@ final class ListLsubCollection implements Serializable {
         }
     }
 
-    private static final Set<String> ATTRIBUTES_NON_EXISTING_PARENT = Collections.unmodifiableSet(new HashSet<String>(Arrays.asList(
-        "\\noselect",
-        "\\haschildren")));
+    private static final Set<String> ATTRIBUTES_NON_EXISTING_PARENT = ImmutableSet.of("\\noselect", "\\haschildren");
 
-    private static final Set<String> ATTRIBUTES_NON_EXISTING_NAMESPACE = Collections.unmodifiableSet(new HashSet<String>(Arrays.asList(
-        "\\noselect",
-        "\\hasnochildren")));
+    private static final Set<String> ATTRIBUTES_NON_EXISTING_NAMESPACE = ImmutableSet.of("\\noselect", "\\hasnochildren");
 
-    // private static final Set<String> ATTRIBUTES_NO_SELECT_NAMESPACE = Collections.unmodifiableSet(new HashSet<String>(Arrays.asList("\\noselect")));
+    // private static final Set<String> ATTRIBUTES_NO_SELECT_NAMESPACE = ImmutableSet.of("\\noselect");
 
     private static final String ATTRIBUTE_DRAFTS = "\\drafts";
     private static final String ATTRIBUTE_JUNK = "\\junk";
     private static final String ATTRIBUTE_SENT = "\\sent";
     private static final String ATTRIBUTE_TRASH = "\\trash";
     private static final String ATTRIBUTE_ARCHIVE = "\\archive";
+    /**
+     * New mailbox attribute added by the "LIST-EXTENDED" extension.
+     */
+    private static final String ATTRIBUTE_NON_EXISTENT = "\\nonexistent";
 
     private static final String[] ATTRIBUTES_SPECIAL_USE = { ATTRIBUTE_DRAFTS, ATTRIBUTE_JUNK, ATTRIBUTE_SENT, ATTRIBUTE_TRASH };
 
@@ -819,14 +922,18 @@ final class ListLsubCollection implements Serializable {
      *
      * @param protocol The IMAP protocol
      * @param lsub <code>true</code> to perform a LSUB command; otherwise <code>false</code> for LIST
+     * @param responses Test responses
      * @throws ProtocolException If a protocol error occurs
      */
-    protected void doListLsubCommand(final IMAPProtocol protocol, final boolean lsub) throws ProtocolException {
+    protected void doListLsubCommand(final IMAPProtocol protocol, final boolean lsub, Response[] responses) throws ProtocolException {
         // Perform command
         String command = lsub ? "LSUB" : "LIST";
         String sCmd = new StringBuilder(command).append(" \"\" \"*\"").toString();
 
-        Response[] r = performCommand(protocol, sCmd);
+        if (protocol == null && responses == null) {
+            throw new IllegalArgumentException("Unable to perform ListLsubCommand wihtout protocol and responses!");
+        }
+        Response[] r = null == responses ? performCommand(protocol, sCmd) : responses;
         LOG.debug("{} cache filled with >>{}<< which returned {} response line(s).", (command), sCmd, Integer.valueOf(r.length));
 
         Response response = r[r.length - 1];
@@ -847,6 +954,7 @@ final class ListLsubCollection implements Serializable {
                 }
                 final String fullName = listLsubEntry.getFullName();
                 final String originalFullName = listLsubEntry.optOriginalFullName();
+
                 // (Re-)Set children
                 {
                     ListLsubEntryImpl oldEntry = map.get(fullName);
@@ -854,14 +962,12 @@ final class ListLsubCollection implements Serializable {
                         // Re-lookup with original full name
                         oldEntry = map.get(originalFullName);
                     }
-
                     if (oldEntry == null) {
                         // Wasn't in map before
                         map.put(fullName, listLsubEntry);
                         if (originalFullName != null) {
                             map.put(originalFullName, listLsubEntry);
                         }
-
                     } else {
                         // Already contained in map
                         oldEntry.clearChildren();
@@ -923,12 +1029,16 @@ final class ListLsubCollection implements Serializable {
             }
 
             // Dispatch remaining untagged responses
-            protocol.notifyResponseHandlers(r);
+            if (null != protocol) {
+                protocol.notifyResponseHandlers(r);
+            }
         } else {
             // Dispatch remaining untagged responses
             LogProperties.putProperty(LogProperties.Name.MAIL_COMMAND, sCmd);
-            protocol.notifyResponseHandlers(r);
-            protocol.handleResult(response);
+            if (protocol != null) {
+                protocol.notifyResponseHandlers(r);
+                protocol.handleResult(response);
+            }
         }
     }
 
@@ -981,7 +1091,10 @@ final class ListLsubCollection implements Serializable {
             if (r[i] instanceof IMAPResponse) {
                 IMAPResponse ir = (IMAPResponse) r[i];
                 if (ir.keyEquals(command)) {
-                    list.add(parseListResponse(ir, lsub ? null : lsubMap));
+                    ListLsubEntryImpl listLsubEntry = parseListResponse(ir, lsub ? null : lsubMap);
+                    if (listLsubEntry != null) {
+                        list.add(listLsubEntry);
+                    }
                     r[i] = null;
                 }
             }
@@ -1072,17 +1185,24 @@ final class ListLsubCollection implements Serializable {
      * Performs a LIST command for root folder with specified IMAP protocol.
      *
      * @param protocol The IMAP protocol
+     * @param responses Test responses
      * @throws ProtocolException If a protocol error occurs
      */
-    protected void doRootListCommand(final IMAPProtocol protocol) throws ProtocolException {
+    protected void doRootListCommand(final IMAPProtocol protocol, Response[] responses) throws ProtocolException {
         /*
          * Perform command: LIST "" ""
          */
         String command = "LIST \"\" \"\"";
-        Response[] r = performCommand(protocol, command);
+        Response[] r = null == responses ? performCommand(protocol, command) : responses;
 
         if (r.length == 1) {
-            // No LIST response for root folder. Do dummy LSUB and retry...
+            // No LIST response for root folder.
+            if (r[0].isBYE()) {
+                LogProperties.putProperty(LogProperties.Name.MAIL_COMMAND, command);
+                protocol.handleResult(r[0]);
+            }
+
+            // Do dummy LSUB and retry...
             doDummyLsub(protocol);
             r = performCommand(protocol, command);
         }
@@ -1123,7 +1243,9 @@ final class ListLsubCollection implements Serializable {
         /*
          * Dispatch remaining untagged responses
          */
-        protocol.notifyResponseHandlers(r);
+        if (null != protocol) {
+            protocol.notifyResponseHandlers(r);
+        }
     }
 
     /**
@@ -1888,6 +2010,10 @@ final class ListLsubCollection implements Serializable {
         // Read full name; decode the name (using RFC2060's modified UTF7)
         listResponse.skipSpaces();
         String name = null == predefinedName ? BASE64MailboxDecoder.decode(listResponse.readAtomString()) : predefinedName;
+        if (MailFolderUtility.isInvalidFullName(name)) {
+            LOG.warn("Detected unsupported full name: {}. IMAP folder will be discarded.", name);
+            return null;
+        }
 
         // Return
         return new ListLsubEntryImpl(name, attributes, separator, changeState, hasInferiors, canOpen, hasChildren, lsubMap).setNamespace(isNamespace(name));
@@ -1953,6 +2079,11 @@ final class ListLsubCollection implements Serializable {
             return false;
         }
 
+        @Override
+        public boolean existsAndIsNotNonExistent() {
+            return false;
+        }
+
         @Override
         public ListLsubEntry getParent() {
             return null;
@@ -1968,11 +2099,6 @@ final class ListLsubCollection implements Serializable {
             return fullName;
         }
 
-        @Override
-        public String optOriginalFullName() {
-            return null;
-        }
-
         @Override
         public Set<String> getAttributes() {
             return Collections.emptySet();
@@ -2048,11 +2174,6 @@ final class ListLsubCollection implements Serializable {
             return false;
         }
 
-        @Override
-        public Rights getMyRights() {
-            return null;
-        }
-
     }
 
     /**
@@ -2072,6 +2193,8 @@ final class ListLsubCollection implements Serializable {
 
         private final String originalFullName;
 
+        private boolean nonExistent;
+
         private Set<String> attributes;
 
         private char separator;
@@ -2092,8 +2215,6 @@ final class ListLsubCollection implements Serializable {
 
         private Boolean hasChildren;
 
-        private Rights myRights;
-
         private Boolean subscribed;
 
         private boolean dummy;
@@ -2102,9 +2223,9 @@ final class ListLsubCollection implements Serializable {
             super();
             String checkedFullName = checkFullName(fullName, separator);
             this.fullName = checkedFullName;
-            this.originalFullName = checkedFullName.length() == 0 /* root full-name is the empty string */ || fullName.equals(checkedFullName) ? null : fullName;
-
+            this.originalFullName = checkedFullName.length() == 0 /*root full-name is the empty string*/ || fullName.equals(checkedFullName) ? null : fullName;
             this.attributes = attributes;
+            this.nonExistent = null != attributes && attributes.contains(ATTRIBUTE_NON_EXISTENT);
             this.separator = separator;
             this.changeState = changeState;
             this.hasInferiors = hasInferiors;
@@ -2127,6 +2248,7 @@ final class ListLsubCollection implements Serializable {
             fullName = newEntry.fullName;
             originalFullName = newEntry.originalFullName;
             attributes = newEntry.attributes;
+            nonExistent = newEntry.nonExistent;
             canOpen = newEntry.canOpen;
             changeState = newEntry.changeState;
             hasInferiors = newEntry.hasInferiors;
@@ -2144,6 +2266,7 @@ final class ListLsubCollection implements Serializable {
                 return;
             }
             attributes = newEntry.attributes;
+            nonExistent = newEntry.nonExistent;
             canOpen = newEntry.canOpen;
             changeState = newEntry.changeState;
             hasInferiors = newEntry.hasInferiors;
@@ -2218,7 +2341,7 @@ final class ListLsubCollection implements Serializable {
                 return;
             }
             if (null == children) {
-                children = new HashSet<ListLsubEntryImpl>(8);
+                children = new ConcurrentHashSet<ListLsubEntryImpl>(8);
                 children.add(child);
             } else {
                 if (!children.add(child)) {
@@ -2350,6 +2473,12 @@ final class ListLsubCollection implements Serializable {
             return true;
         }
 
+        @Override
+        public boolean existsAndIsNotNonExistent() {
+            // This instance is known to exist, therefore only check for \NonExistent attribute
+            return !nonExistent;
+        }
+
         protected void setSubscribed(final boolean subscribed) {
             this.subscribed = Boolean.valueOf(subscribed);
         }
@@ -2359,20 +2488,6 @@ final class ListLsubCollection implements Serializable {
             return null == subscribed ? (null == lsubMap ? true : lsubMap.containsKey(fullName)) : subscribed.booleanValue();
         }
 
-        /**
-         * Sets the status.
-         *
-         * @param status The status
-         */
-        protected void setStatus(final int[] status) {
-            if (null == status) {
-                this.status = null;
-                return;
-            }
-            this.status = new int[status.length];
-            System.arraycopy(status, 0, this.status, 0, status.length);
-        }
-
         @Override
         public int getMessageCount() {
             return null == status ? -1 : status[0];
@@ -2388,29 +2503,6 @@ final class ListLsubCollection implements Serializable {
             return null == status ? -1 : status[2];
         }
 
-        protected void setMyRights(final Rights myRights) {
-            this.myRights = myRights;
-        }
-
-        /**
-         * Gets MYRIGHTS.
-         *
-         * @return MYRIGHTS or <code>null</code> if absent
-         */
-        @Override
-        public Rights getMyRights() {
-            return myRights;
-        }
-
-        /**
-         * Sets the ACLs.
-         *
-         * @param acls The ACL list
-         */
-        protected void setAcls(final List<ACL> acls) {
-            this.acls = acls;
-        }
-
         @Override
         public List<ACL> getACLs() {
             return acls == null ? null : new ArrayList<ACL>(acls);
@@ -2516,35 +2608,6 @@ final class ListLsubCollection implements Serializable {
 
     } // End of class ListLsubEntryImpl
 
-    private static void appendStackTrace(final StackTraceElement[] trace, final StringBuilder sb, final String lineSeparator) {
-        if (null == trace) {
-            sb.append("<missing stack trace>\n");
-            return;
-        }
-        for (final StackTraceElement ste : trace) {
-            final String className = ste.getClassName();
-            if (null != className) {
-                sb.append("    at ").append(className).append('.').append(ste.getMethodName());
-                if (ste.isNativeMethod()) {
-                    sb.append("(Native Method)");
-                } else {
-                    final String fileName = ste.getFileName();
-                    if (null == fileName) {
-                        sb.append("(Unknown Source)");
-                    } else {
-                        final int lineNumber = ste.getLineNumber();
-                        sb.append('(').append(fileName);
-                        if (lineNumber >= 0) {
-                            sb.append(':').append(lineNumber);
-                        }
-                        sb.append(')');
-                    }
-                }
-                sb.append(lineSeparator);
-            }
-        }
-    }
-
     /** Checks the full name */
     protected static String checkFullName(final String fullName, final char separator) {
         if (null == fullName) {
@@ -2562,4 +2625,5 @@ final class ListLsubCollection implements Serializable {
         }
         return fullName;
     }
+
 }
