@@ -49,12 +49,12 @@
 
 package com.openexchange.imap.command;
 
+import static com.openexchange.java.Autoboxing.I;
 import java.io.IOException;
 import java.io.InputStream;
 import java.text.ParseException;
 import java.util.Collection;
 import java.util.Enumeration;
-import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Map;
 import java.util.Set;
@@ -110,7 +110,8 @@ import gnu.trove.map.hash.TLongObjectHashMap;
  */
 public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjectMap<MailMessage>> {
 
-    private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(SimpleFetchIMAPCommand.class);
+    /** The logger constant */
+    static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(SimpleFetchIMAPCommand.class);
 
     private static final int LENGTH = 9; // "FETCH <nums> (<command>)"
 
@@ -136,9 +137,10 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
      * @param fp The fetch profile to use
      * @param serverInfo The IMAP server information
      * @param examineHasAttachmentUserFlags Whether has-attachment user flags should be considered
+     * @param previewSupported Whether target IMAP server supports <code>"PREVIEW=FUZZY"</code> capability
      * @throws MessagingException If initialization fails
      */
-    public SimpleFetchIMAPCommand(IMAPFolder imapFolder, boolean isRev1, int[] seqNums, FetchProfile fp, IMAPServerInfo serverInfo, boolean examineHasAttachmentUserFlags) throws MessagingException {
+    public SimpleFetchIMAPCommand(IMAPFolder imapFolder, boolean isRev1, int[] seqNums, FetchProfile fp, IMAPServerInfo serverInfo, boolean examineHasAttachmentUserFlags, boolean previewSupported) throws MessagingException {
         super(imapFolder);
         final int messageCount = imapFolder.getMessageCount();
         if (messageCount <= 0) {
@@ -146,11 +148,11 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
         }
         this.examineHasAttachmentUserFlags = examineHasAttachmentUserFlags;
         lastHandlers = new HashSet<FetchItemHandler>();
-        command = getFetchCommand(isRev1, fp, false, serverInfo);
+        command = getFetchCommand(isRev1, fp, false, serverInfo, previewSupported);
         uid = false;
         length = seqNums.length;
         map = new TLongObjectHashMap<MailMessage>(length);
-        args = length == messageCount ? (1 == length ? new String[] { "1" } : ARGS_ALL) : IMAPNumArgSplitter.splitSeqNumArg(seqNums, false, LENGTH + command.length());
+        args = length == messageCount ? (1 == length ? ARGS_FIRST : ARGS_ALL) : IMAPNumArgSplitter.splitSeqNumArg(seqNums, false, LENGTH + command.length());
         if (0 == length) {
             returnDefaultValue = true;
         }
@@ -166,9 +168,10 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
      * @param fetchProfile The fetch profile to use
      * @param serverInfo The IMAP server information
      * @param examineHasAttachmentUserFlags Whether has-attachment user flags should be considered
+     * @param previewSupported Whether target IMAP server supports <code>"PREVIEW=FUZZY"</code> capability
      * @throws MessagingException If initialization fails
      */
-    public SimpleFetchIMAPCommand(IMAPFolder imapFolder, boolean isRev1, long[] uids, FetchProfile fetchProfile, IMAPServerInfo imapServerInfo, boolean examineHasAttachmentUserFlags) throws MessagingException {
+    public SimpleFetchIMAPCommand(IMAPFolder imapFolder, boolean isRev1, long[] uids, FetchProfile fetchProfile, IMAPServerInfo imapServerInfo, boolean examineHasAttachmentUserFlags, boolean previewSupported) throws MessagingException {
         super(imapFolder);
         final int messageCount = imapFolder.getMessageCount();
         if (messageCount <= 0) {
@@ -180,11 +183,11 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
         map = new TLongObjectHashMap<MailMessage>(length);
         if (length == messageCount) {
             fetchProfile.add(UIDFolder.FetchProfileItem.UID);
-            command = getFetchCommand(isRev1, fetchProfile, false, imapServerInfo);
-            args = (1 == length ? new String[] { "1" } : ARGS_ALL);
+            command = getFetchCommand(isRev1, fetchProfile, false, imapServerInfo, previewSupported);
+            args = (1 == length ? ARGS_FIRST : ARGS_ALL);
             uid = false;
         } else {
-            command = getFetchCommand(isRev1, fetchProfile, false, imapServerInfo);
+            command = getFetchCommand(isRev1, fetchProfile, false, imapServerInfo, previewSupported);
             args = IMAPNumArgSplitter.splitUIDArg(uids, false, LENGTH_WITH_UID + command.length());
             uid = true;
         }
@@ -277,7 +280,7 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
         if (uid) {
             UID item = getItemOf(UID.class, fetchResponse);
             if (item == null) {
-                LOG.warn("Message #{} discarded", seqNum, IMAPException.IMAPCode.UNEXPECTED_ERROR.create("Unable to retrieve UID from response."));
+                LOG.warn("Message #{} discarded", I(seqNum), IMAPException.IMAPCode.UNEXPECTED_ERROR.create("Unable to retrieve UID from response."));
                 return true;
             }
             id = item.uid;
@@ -342,14 +345,14 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
              */
             {
                 final OXException imapExc = MimeMailException.handleMessagingException(e);
-                LOG.warn("Message #{} discarded", mail.getSeqnum(), imapExc);
+                LOG.warn("Message #{} discarded", I(mail.getSeqnum()), imapExc);
             }
             error = true;
         } catch (final OXException e) {
             /*
              * Discard corrupt message
              */
-            LOG.warn("Message #{} discarded", mail.getSeqnum(), e);
+            LOG.warn("Message #{} discarded", I(mail.getSeqnum()), e);
             error = true;
         }
         if (!error) {
@@ -400,8 +403,6 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
         void handleMessage(final Message message, final IDMailMessage msg, final org.slf4j.Logger logger) throws MessagingException, OXException;
     }
 
-    private static final String MULTI_SUBTYPE_MIXED = "MIXED";
-
     /*-
      * ++++++++++++++ Item handlers ++++++++++++++
      */
@@ -414,93 +415,90 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
 
     private static final FetchItemHandler HEADER_ITEM_HANDLER = new FetchItemHandler() {
 
-        private final Map<String, HeaderHandler> hh = new HashMap<String, HeaderHandler>() {
+        private final Map<String, HeaderHandler> hh = ImmutableMap.<String, SimpleFetchIMAPCommand.HeaderHandler> builder()
+            .put(MessageHeaders.HDR_FROM, new HeaderHandler() {
 
-            {
-                put(MessageHeaders.HDR_FROM, new HeaderHandler() {
+                @Override
+                public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
+                    mailMessage.addFrom(MimeMessageUtility.getAddressHeader(hdr.getValue()));
+                }
+            })
+            .put(MessageHeaders.HDR_TO, new HeaderHandler() {
 
-                    @Override
-                    public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
-                        mailMessage.addFrom(MimeMessageUtility.getAddressHeader(hdr.getValue()));
-                    }
-                });
-                put(MessageHeaders.HDR_TO, new HeaderHandler() {
+                @Override
+                public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
+                    mailMessage.addTo(MimeMessageUtility.getAddressHeader(hdr.getValue()));
+                }
+            })
+            .put(MessageHeaders.HDR_CC, new HeaderHandler() {
 
-                    @Override
-                    public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
-                        mailMessage.addTo(MimeMessageUtility.getAddressHeader(hdr.getValue()));
-                    }
-                });
-                put(MessageHeaders.HDR_CC, new HeaderHandler() {
+                @Override
+                public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
+                    mailMessage.addCc(MimeMessageUtility.getAddressHeader(hdr.getValue()));
+                }
+            })
+            .put(MessageHeaders.HDR_BCC, new HeaderHandler() {
 
-                    @Override
-                    public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
-                        mailMessage.addCc(MimeMessageUtility.getAddressHeader(hdr.getValue()));
-                    }
-                });
-                put(MessageHeaders.HDR_BCC, new HeaderHandler() {
+                @Override
+                public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
+                    mailMessage.addBcc(MimeMessageUtility.getAddressHeader(hdr.getValue()));
+                }
+            })
+            .put(MessageHeaders.HDR_REPLY_TO, new HeaderHandler() {
 
-                    @Override
-                    public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
-                        mailMessage.addBcc(MimeMessageUtility.getAddressHeader(hdr.getValue()));
-                    }
-                });
-                put(MessageHeaders.HDR_REPLY_TO, new HeaderHandler() {
+                @Override
+                public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
+                    mailMessage.addReplyTo(MimeMessageUtility.getAddressHeader(hdr.getValue()));
+                }
+            })
+            .put(MessageHeaders.HDR_DISP_NOT_TO, new HeaderHandler() {
 
-                    @Override
-                    public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
-                        mailMessage.addReplyTo(MimeMessageUtility.getAddressHeader(hdr.getValue()));
-                    }
-                });
-                put(MessageHeaders.HDR_DISP_NOT_TO, new HeaderHandler() {
+                @Override
+                public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
+                    mailMessage.setDispositionNotification(MimeMessageUtility.getAddressHeader(hdr.getValue())[0]);
+                }
+            })
+            .put(MessageHeaders.HDR_SUBJECT, new HeaderHandler() {
 
-                    @Override
-                    public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
-                        mailMessage.setDispositionNotification(MimeMessageUtility.getAddressHeader(hdr.getValue())[0]);
-                    }
-                });
-                put(MessageHeaders.HDR_SUBJECT, new HeaderHandler() {
+                @Override
+                public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
+                    mailMessage.setSubject(MimeMessageUtility.decodeMultiEncodedHeader(MimeMessageUtility.checkNonAscii(hdr.getValue())), true);
+                }
+            })
+            .put(MessageHeaders.HDR_DATE, new HeaderHandler() {
 
-                    @Override
-                    public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
-                        mailMessage.setSubject(MimeMessageUtility.decodeMultiEncodedHeader(MimeMessageUtility.checkNonAscii(hdr.getValue())), true);
-                    }
-                });
-                put(MessageHeaders.HDR_DATE, new HeaderHandler() {
-
-                    @Override
-                    public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
-                        final MailDateFormat mdf = MimeMessageUtility.getDefaultMailDateFormat();
-                        synchronized (mdf) {
-                            try {
-                                mailMessage.setSentDate(mdf.parse(hdr.getValue()));
-                            } catch (final ParseException e) {
-                                LOG.error("", e);
-                            }
+                @Override
+                public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
+                    final MailDateFormat mdf = MimeMessageUtility.getDefaultMailDateFormat();
+                    synchronized (mdf) {
+                        try {
+                            mailMessage.setSentDate(mdf.parse(hdr.getValue()));
+                        } catch (final ParseException e) {
+                            LOG.error("", e);
                         }
                     }
-                });
-                put(MessageHeaders.HDR_IMPORTANCE, new HeaderHandler() {
+                }
+            })
+            .put(MessageHeaders.HDR_IMPORTANCE, new HeaderHandler() {
 
-                    @Override
-                    public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
-                        final String value = hdr.getValue();
-                        if (null != value) {
-                            mailMessage.setPriority(MimeMessageUtility.parseImportance(value));
-                        }
+                @Override
+                public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
+                    final String value = hdr.getValue();
+                    if (null != value) {
+                        mailMessage.setPriority(MimeMessageUtility.parseImportance(value));
                     }
-                });
-                put(MessageHeaders.HDR_X_PRIORITY, new HeaderHandler() {
+                }
+            })
+            .put(MessageHeaders.HDR_X_PRIORITY, new HeaderHandler() {
 
-                    @Override
-                    public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
-                        if (!mailMessage.containsPriority()) {
-                            mailMessage.setPriority(MimeMessageUtility.parsePriority(hdr.getValue()));
-                        }
+                @Override
+                public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
+                    if (!mailMessage.containsPriority()) {
+                        mailMessage.setPriority(MimeMessageUtility.parsePriority(hdr.getValue()));
                     }
-                });
-            }
-        };
+                }
+            })
+            .build();
 
         @Override
         public void handleItem(final Item item, final IDMailMessage msg, final org.slf4j.Logger logger) throws MessagingException, OXException {
@@ -520,7 +518,7 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
                 }
                 h = new InternetHeaders();
                 if (null == headerStream) {
-                    logger.debug("Cannot retrieve headers from message #{} in folder {}", msg.getSeqnum(), msg.getFolder());
+                    logger.debug("Cannot retrieve headers from message #{} in folder {}", I(msg.getSeqnum()), msg.getFolder());
                 } else {
                     h.load(headerStream);
                 }
@@ -629,7 +627,7 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
                     if (MailMessage.isColorLabel(userFlag)) {
                         try {
                             colorLabel = MailMessage.getColorLabelIntValue(userFlag);
-                        } catch (final OXException e) {
+                        } catch (@SuppressWarnings("unused") final OXException e) {
                             // Cannot occur
                             colorLabel = MailMessage.COLOR_LABEL_NONE;
                         }
@@ -697,7 +695,7 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
                     if (MailMessage.isColorLabel(userFlag)) {
                         try {
                             colorLabel = MailMessage.getColorLabelIntValue(userFlag);
-                        } catch (final OXException e) {
+                        } catch (@SuppressWarnings("unused") final OXException e) {
                             // Cannot occur
                             colorLabel = MailMessage.COLOR_LABEL_NONE;
                         }
@@ -817,7 +815,7 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
             String contentType;
             try {
                 contentType = message.getContentType();
-            } catch (final MessagingException e) {
+            } catch (@SuppressWarnings("unused") final MessagingException e) {
                 final String[] header = message.getHeader("Content-Type");
                 if (null != header && header.length > 0) {
                     contentType = header[0];
@@ -923,10 +921,11 @@ public final class SimpleFetchIMAPCommand extends AbstractIMAPCommand<TLongObjec
      * @param fp The fetch profile to convert
      * @param loadBody <code>true</code> if message body should be loaded; otherwise <code>false</code>
      * @param serverInfo The IMAP server information
+     * @param previewSupported Whether target IMAP server supports <code>"PREVIEW=FUZZY"</code> capability
      * @return The FETCH items to craft a FETCH command
      */
-    private static String getFetchCommand(boolean isRev1, FetchProfile fp, boolean loadBody, IMAPServerInfo serverInfo) {
-        return MailMessageFetchIMAPCommand.getFetchCommand(isRev1, fp, loadBody, serverInfo);
+    private static String getFetchCommand(boolean isRev1, FetchProfile fp, boolean loadBody, IMAPServerInfo serverInfo, boolean previewSupported) {
+        return MailMessageFetchIMAPCommand.getFetchCommand(isRev1, fp, loadBody, serverInfo, previewSupported);
     }
 
     /**
