@@ -68,8 +68,8 @@ public enum IMAPServer {
     COURIER("Courier", CourierEntity2ACL.getInstance(), new ArgumentGenerator() {
 
         @Override
-        public Object[] getArguments(int accountId, String serverUrl, int sessionUser, String fullname, char separator, String otherUserNamespace, String publicNamespaces) {
-            return new Object[] { Integer.valueOf(accountId), serverUrl, Integer.valueOf(sessionUser), fullname, Character.valueOf(separator), otherUserNamespace, publicNamespaces };
+        public Object[] getArguments(int accountId, String serverUrl, int sessionUser, String fullname, char separator, String[] otherUserNamespaces, String[] publicNamespaces) {
+            return new Object[] { Integer.valueOf(accountId), serverUrl, Integer.valueOf(sessionUser), fullname, Character.valueOf(separator), otherUserNamespaces, publicNamespaces };
         }
     }, new GreetingMatcher() {
 
@@ -84,7 +84,7 @@ public enum IMAPServer {
     CYRUS("Cyrus", CyrusEntity2ACL.getInstance(), new ArgumentGenerator() {
 
         @Override
-        public Object[] getArguments(int accountId, String serverUrl, int sessionUser, String fullname, char separator, String otherUserNamespace, String publicNamespaces) {
+        public Object[] getArguments(int accountId, String serverUrl, int sessionUser, String fullname, char separator, String[] otherUserNamespaces, String[] publicNamespaces) {
             return new Object[] { Integer.valueOf(accountId), serverUrl, Integer.valueOf(sessionUser) };
         }
     }, new GreetingMatcher() {
@@ -100,8 +100,8 @@ public enum IMAPServer {
     DOVECOT("Dovecot", DovecotEntity2ACL.getInstance(), new ArgumentGenerator() {
 
         @Override
-        public Object[] getArguments(int accountId, String serverUrl, int sessionUser, String fullname, char separator, String otherUserNamespace, String publicNamespaces) {
-            return new Object[] { Integer.valueOf(accountId), serverUrl, Integer.valueOf(sessionUser), fullname, Character.valueOf(separator), otherUserNamespace, publicNamespaces };
+        public Object[] getArguments(int accountId, String serverUrl, int sessionUser, String fullname, char separator, String[] otherUserNamespaces, String[] publicNamespaces) {
+            return new Object[] { Integer.valueOf(accountId), serverUrl, Integer.valueOf(sessionUser), fullname, Character.valueOf(separator), otherUserNamespaces, publicNamespaces };
         }
     }, new GreetingMatcher() {
 
@@ -116,7 +116,7 @@ public enum IMAPServer {
     SUN_MESSAGING_SERVER("Sun", SUNMessagingServerEntity2ACL.getInstance(), new ArgumentGenerator() {
 
         @Override
-        public Object[] getArguments(int accountId, String serverUrl, int sessionUser, String fullname, char separator, String otherUserNamespace, String publicNamespaces) {
+        public Object[] getArguments(int accountId, String serverUrl, int sessionUser, String fullname, char separator, String[] otherUserNamespaces, String[] publicNamespaces) {
             return new Object[] { Integer.valueOf(accountId), serverUrl, Integer.valueOf(sessionUser) };
         }
     }, new GreetingMatcher() {
@@ -132,7 +132,7 @@ public enum IMAPServer {
     MDAEMON("MDaemon", MDaemonEntity2ACL.getInstance(), new ArgumentGenerator() {
 
         @Override
-        public Object[] getArguments(int accountId, String serverUrl, int sessionUser, String fullname, char separator, String otherUserNamespace, String publicNamespaces) {
+        public Object[] getArguments(int accountId, String serverUrl, int sessionUser, String fullname, char separator, String[] otherUserNamespaces, String[] publicNamespaces) {
             return new Object[] { Integer.valueOf(accountId), serverUrl, Integer.valueOf(sessionUser) };
         }
     }, new GreetingMatcher() {
@@ -184,11 +184,11 @@ public enum IMAPServer {
      * @param fullname The IMAP folder's full name
      * @param separator The IMAP folder's separator
      * @param otherUserNamespace The user's shared namespace; e.g. <code>"shared"</code>
-     * @param
+     * @param publicNamespaces The public namespaces
      * @return The needed arguments to reliably map an ACL entity to a system user and vice versa
      */
-    public Object[] getArguments(int accountId, String serverUrl, int sessionUser, String fullname, char separator, String otherUserNamespace, String publicNamespaces) {
-        return argumentGenerator.getArguments(accountId, serverUrl, sessionUser, fullname, separator, otherUserNamespace, publicNamespaces);
+    public Object[] getArguments(int accountId, String serverUrl, int sessionUser, String fullname, char separator, String[] otherUserNamespaces, String[] publicNamespaces) {
+        return argumentGenerator.getArguments(accountId, serverUrl, sessionUser, fullname, separator, otherUserNamespaces, publicNamespaces);
     }
 
     /**
@@ -245,7 +245,7 @@ public enum IMAPServer {
          * @param publicNamespaces The public namespace
          * @return The arguments needed for a certain IMAP server to map ACL entity to a system user and vice versa
          */
-        public Object[] getArguments(int accountId, String serverUrl, int sessionUser, String fullname, char separator, String otherUserNamespace, String publicNamespaces);
+        public Object[] getArguments(int accountId, String serverUrl, int sessionUser, String fullname, char separator, String[] otherUserNamespaces, String[] publicNamespaces);
     }
 
     private static abstract class GreetingMatcher {
@@ -278,7 +278,7 @@ public enum IMAPServer {
          * @param greeting The IMAP server greeting
          * @return <code>true</code> if given IMAP server greeting matches a certain IMAP server; otherwise <code>false</code>
          */
-        public abstract boolean matches(String greeting);
+        abstract boolean matches(String greeting);
     }
 
 }
